/*====================================================================
 *
 * 		Portable Bark Killer v1.0
 * 		2013 by blankfield <blank0field@gmail.com>
 * 		MSP430G2553 @ 16MHz, 3,3V
 * 		ANAREN RF MODULE
 *
 ===================================================================*/

/*====================================================================
 *
 * headers and includes
 *
 ===================================================================*/
#include <msp430.h>
#include <ti/mcu/msp430/Grace.h>
#include "main.h"

/*====================================================================
 *
 * constants and definitions
 *
 ===================================================================*/
#define ButtonON	BIT3	//P2.3 - LOCAL ON BUTTON
#define Vbattery	BIT7	//P1.7 - F1
#define PWM			BIT1	//P2.1 - F2
#define LED			BIT2	//P2.2 - LED indicator
#define Vref        (float)1.5
#define ADCRES      (float)1023
#define R1resistor  (float)334200 //330kR
#define R2resistor  (float)99820  //100kR
#define R3resistor  (float)10000 //10kR
#define Uconstant   (float)((Vref/ADCRES)*((R1resistor+R2resistor)/R2resistor))
enum    enumDAYorNIGHT	 		{DAY, NIGHT};
enum    enumCommands	 		{GET_STATS, GET_SUNLVL, GET_VBATT,
								 GET_MODE, SET_MODE};


#define GDO0_IN         P1IN
#define GDO0_OUT        P1OUT
#define GDO0_DIR        P1DIR
#define GDO0_IE         P1IE
#define GDO0_IES        P1IES
#define GDO0_IFG        P1IFG
#define GDO0_PIN        BIT4
#define GDO1_IN         P1IN
#define GDO1_OUT        P1OUT
#define GDO1_DIR        P1DIR
#define GDO1_PIN          BIT6
#define GDO2_IN         P1IN
#define GDO2_OUT        P1OUT
#define GDO2_DIR        P1DIR
#define GDO2_PIN        BIT3
#define CSn_OUT         P1OUT
#define CSn_DIR         P1DIR
#define CSn_PIN           BIT0
#define SPI_UB0_SEL   P1SEL
#define SPI_UB0_SEL2  P1SEL2
#define SPI_UB0_DIR   P1DIR
#define SPI_UB0_IN    P1IN
#define SPI_UB0_SIMO    BIT7
#define SPI_UB0_SOMI    BIT6
#define SPI_UB0_UCLK    BIT5








/*====================================================================
 *
 * variables
 *
 ===================================================================*/
char MODE;
char STATS;
float VBATT;
int SUNLVL;
char DAYorNIGHT;

//unsigned char RFcommandMainLight[CSLCOMMANDLENGTH]={0x2C, 0x96, 0xD9, 0x64, 0x92};
// Deviation = 47.607422
// Base frequency = 868.349854
// Carrier frequency = 868.349854
// Channel number = 0
// Carrier frequency = 868.349854
// Modulation format = ASK/OOK
// Manchester enable = false
// Sync word qualifier mode = No preamble/sync
// Preamble count = 4
// Channel spacing = 199.951172
// Carrier frequency = 868.349854
// Data rate = 1.59931
// RX filter BW = 203.125000
// Data format = Normal mode
// Length config = Fixed packet length mode. Length configured in PKTLEN register
// CRC enable = false
// Packet length = 50
// Device address = 0
// Address config = No address check
// CRC autoflush = false
// PA ramping = false
// TX power = 0




// RF settings for CC110L
typedef struct RF_SETTINGS {
    char iocfg2;          // GDO2 Output Pin Configuration
    char iocfg0;          // GDO0 Output Pin Configuration
    char fifothr;         // RX FIFO and TX FIFO Thresholds
    char pktctrl0;        // Packet Automation Control
    char fsctrl1;         // Frequency Synthesizer Control
    char freq2;           // Frequency Control Word, High Byte
    char freq1;           // Frequency Control Word, Middle Byte
    char freq0;           // Frequency Control Word, Low Byte
    char mdmcfg4;         // Modem Configuration
    char mdmcfg3;         // Modem Configuration
    char mdmcfg2;         // Modem Configuration
    char deviatn;         // Modem Deviation Setting
    char mcsm0;           // Main Radio Control State Machine Configuration
    char foccfg;          // Frequency Offset Compensation Configuration
    char reserved_0x20;   // Use setting from SmartRF Studio
    char frend0;          // Front End TX Configuration
    char fscal3;          // Frequency Synthesizer Calibration
    char fscal2;          // Frequency Synthesizer Calibration
    char fscal1;          // Frequency Synthesizer Calibration
    char fscal0;          // Frequency Synthesizer Calibration
    char test2;           // Various Test Settings
    char test1;           // Various Test Settings
    char test0;           // Various Test Settings
}RF_SETTINGS;

RF_SETTINGS rfSettings = {
    0x06,  // IOCFG0             GDO0 Output Pin Configuration
    0x47,  // FIFOTHR            RX FIFO and TX FIFO Thresholds
    0x05,  // PKTCTRL0           Packet Automation Control
    0x06,  // FSCTRL1            Frequency Synthesizer Control
    0x21,  // FREQ2              Frequency Control Word, High Byte
    0x65,  // FREQ1              Frequency Control Word, Middle Byte
    0xE8,  // FREQ0              Frequency Control Word, Low Byte
    0xF6,  // MDMCFG4            Modem Configuration
    0x02,  // MDMCFG3            Modem Configuration
    0x33,  // MDMCFG2            Modem Configuration
    0x15,  // DEVIATN            Modem Deviation Setting
    0x18,  // MCSM0              Main Radio Control State Machine Configuration
    0x16,  // FOCCFG             Frequency Offset Compensation Configuration
    0xFB,  // RESERVED_0X20      Use setting from SmartRF Studio
    0x11,  // FREND0             Front End TX Configuration
    0xE9,  // FSCAL3             Frequency Synthesizer Calibration
    0x2A,  // FSCAL2             Frequency Synthesizer Calibration
    0x00,  // FSCAL1             Frequency Synthesizer Calibration
    0x1F,  // FSCAL0             Frequency Synthesizer Calibration
    0x81,  // TEST2              Various Test Settings
    0x35,  // TEST1              Various Test Settings
    0x09,  // TEST0              Various Test Settings
};

#define    CC110L_IOCFG2           0x0b
#define    CC110L_IOCFG1           0x2e
#define    CC110L_IOCFG0           0x0c
#define    CC110L_FIFOTHR          0x47
#define    CC110L_SYNC1            0xd3
#define    CC110L_SYNC0            0x91
#define    CC110L_PKTLEN           0xff
#define    CC110L_PKTCTRL1         0x04
#define    CC110L_PKTCTRL0         0x12
#define    CC110L_ADDR             0x00
#define    CC110L_CHANNR           0x00
#define    CC110L_FSCTRL1          0x06
#define    CC110L_FSCTRL0          0x00
#define    CC110L_FREQ2            0x21
#define    CC110L_FREQ1            0x65
#define    CC110L_FREQ0            0xe8
#define    CC110L_MDMCFG4          0xf6
#define    CC110L_MDMCFG3          0x02
#define    CC110L_MDMCFG2          0x30
#define    CC110L_MDMCFG1          0x22
#define    CC110L_MDMCFG0          0xf8
#define    CC110L_DEVIATN          0x15
#define    CC110L_MCSM2            0x07
#define    CC110L_MCSM1            0x30
#define    CC110L_MCSM0            0x18
#define    CC110L_FOCCFG           0x16
#define    CC110L_BSCFG            0x6c
#define    CC110L_AGCCTRL2         0x03
#define    CC110L_AGCCTRL1         0x40
#define    CC110L_AGCCTRL0         0x91
#define    CC110L_RESERVED_0X20    0xfb
#define    CC110L_FREND1           0x56
#define    CC110L_FREND0           0x11
#define    CC110L_FSCAL3           0xe9
#define    CC110L_FSCAL2           0x2a
#define    CC110L_FSCAL1           0x00
#define    CC110L_FSCAL0           0x1f
#define    CC110L_RESERVED_0X29    0x89
#define    CC110L_RESERVED_0X2A    0x127
#define    CC110L_RESERVED_0X2B    0x63
#define    CC110L_TEST2            0x81
#define    CC110L_TEST1            0x35
#define    CC110L_TEST0            0x09
#define    CC110L_PARTNUM          0x00
#define    CC110L_VERSION          0x07
#define    CC110L_FREQEST          0x00
#define    CC110L_CRC_REG          0x00
#define    CC110L_RSSI             0x00
#define    CC110L_MARCSTATE        0x00
#define    CC110L_PKTSTATUS        0x00
#define    CC110L_TXBYTES          0x00
#define    CC110L_RXBYTES          0x00

#define CC110L_WRITE_BURST  0x40
#define CC110L_READ_SINGLE  0x80
#define CC110L_READ_BURST   0xC0
//Masks for appended status bytes
#define CC110L_LQI_RX       0x01        // Position of LQI byte
#define CC110L_CRC_OK       0x80        // Mask "CRC_OK" bit within LQI byte


// Strobe commands
#define CC110L_SRES         0x30        // Reset chip.
#define CC110L_SFSTXON      0x31        // Enable/calibrate freq synthesizer
#define CC110L_SXOFF        0x32        // Turn off crystal oscillator.
#define CC110L_SCAL         0x33        // Calibrate freq synthesizer & disable
#define CC110L_SRX          0x34        // Enable RX.
#define CC110L_STX          0x35        // Enable TX.
#define CC110L_SIDLE        0x37        //
#define CC110L_RESERVED     0x36        // Exit RX / TX
#define CC110L_SPWD         0x39        // Enter pwr down mode when CSn goes hi
#define CC110L_SFRX         0x3A        // Flush the RX FIFO buffer.
#define CC110L_SFTX         0x3B        // Flush the TX FIFO buffer.
#define CC110L_SNOP         0x3D        // No operation.
// Status registers
#define CC110L_PARTNUM      0x30      // Part number
#define CC110L_VERSION      0x31      // Current version number
#define CC110L_FREQEST      0x32      // Frequency offset estimate
#define CC110L_CRC_REG      0x33      // CRC OK
#define CC110L_RSSI         0x34      // Received signal strength indication
#define CC110L_MARCSTATE    0x35      // Control state machine state
#define CC110L_RESERVED     0x36      //
#define CC110L_RESERVED     0x37      //
#define CC110L_PKTSTATUS    0x38      // Current GDOx status and packet status
#define CC110L_RESERVED     0x39      //
#define CC110L_TXBYTES      0x3A      // Underflow and # of bytes in TXFIFO
#define CC110L_RXBYTES      0x3B      // Overflow and # of bytes in RXFIFO
#define CC110L_NUM_RXBYTES  0x7F      // Mask "# of bytes" field in _RXBYTES
// Other memory locations
#define CC110L_PATABLE      0x3E
#define CC110L_TXFIFO       0x3F
#define CC110L_RXFIFO       0x3F

/*====================================================================
 *
 * main()
 *
 ===================================================================*/
int main(void)
{
    Grace_init();// Activate Grace-generated configuration
    while(1){
    	_BIS_SR(LPM0_bits + GIE);
    	_no_operation();
    }
}


/*====================================================================
 *
 * PortInterrupt()
 *
 ===================================================================*/
void PortInterrupt(void){
    while(1){
    	if (!(P2IN & BIT3)){
    		TA1CCR1 = 500;
    		P2OUT |= BIT2;
    	}
    	if ((P2IN & BIT3)){
    		TA1CCR1 = 0;
    		P2OUT &= ~BIT2;
    	}
    }
}

/*====================================================================
 *
 * AutoTrigger()
 * DESC: Interrupt from analog comparator (microphone)
 *
 ===================================================================*/
void AutoTrigger(void){
	CACTL1 &= ~(CAIE);
	TA1CCR1 = 500;
	P2OUT |= BIT2;
	__delay_cycles(50000000);
	P2OUT &= ~BIT2;
	TA1CCR1 = 0;
	__delay_cycles(2000000);

	STATS++;
	CACTL1 &= ~(CAIFG);
	CACTL1 |= (CAIE);
}

/*====================================================================
 *
 * UartRxInterrupt()
 *
 ===================================================================*/
void UartRxInterrupt(void){
	IE2 &= ~(UCA0RXIE);
	__bis_SR_register(GIE);
	switch(UCA0RXBUF){
	case GET_STATS:
		while (!(IFG2 & UCA0TXIFG));
		UCA0TXBUF = STATS;
		break;
	case GET_VBATT:
		while (!(IFG2 & UCA0TXIFG));
		ADC10CTL1 |= INCH2;
		ADC10CTL1 &= ~(INCH3+INCH1+INCH0);
		ADC10CTL0 |= (ADC10SC);
		while ((ADC10CTL0 & ADC10IFG) == 0);
		VBATT = (ADC10MEM * Uconstant);
		UCA0TXBUF = VBATT;
		break;
	case GET_MODE:
		while (!(IFG2 & UCA0TXIFG));
		UCA0TXBUF = MODE;
		break;
	case SET_MODE:
		while (!(IFG2 & UCA0RXIFG));
		MODE = UCA0RXBUF;
		break;
	case GET_SUNLVL:
		/*while (!(IFG2 & UCA0TXIFG));
		ADC10CTL1 |= INCH1+INCH0;
		ADC10CTL1 &= ~(INCH3+INCH2);
		ADC10CTL0 |= (ADC10SC);
		while ((ADC10CTL0 & ADC10IFG) == 0);
		SUNLVL = ((ADC10MEM>>2));*/
		UCA0TXBUF = SUNLVL;
		break;
	}
	IE2 |= (UCA0RXIE);
}



void Wait(unsigned int cycles)

{

  while(cycles>15)                          // 15 cycles consumed by overhead

    cycles = cycles - 6;                    // 6 cycles consumed each iteration

}



void SPISetup(void)

{

  SPI_UB0_SEL |= SPI_UB0_SIMO| SPI_UB0_SOMI| SPI_UB0_UCLK;

  SPI_UB0_SEL2 |= SPI_UB0_SIMO| SPI_UB0_SOMI| SPI_UB0_UCLK;

                                            // SPI option select

  SPI_UB0_DIR |= SPI_UB0_SIMO | SPI_UB0_UCLK;

  CSn_OUT |= CSn_PIN;

  CSn_DIR |= CSn_PIN;         // /CS disable



  UCB0CTL1 |= UCSWRST;                      // **Disable USCI state machine**

  UCB0CTL0 |= UCMODE_0+UCMST+UCCKPH+UCMSB+UCSYNC;    // 3-pin, 8-bit SPI master

  UCB0CTL1 |= UCSSEL_2;                     // SMCLK

  UCB0BR0 = 1 && 0xff;				//0x02;              // UCLK/2

  UCB0BR1 = 1 >> 8;

                                              // SPI TXD out direction

  UCB0CTL1 &= ~UCSWRST;                     // **Initialize USCI state machine**

}



void SPIWriteReg(char addr, char value)

{

  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = addr;                         // Send address

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = value;                        // Send data

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  CSn_OUT |= CSn_PIN;         // /CS disable

}



void SPIWriteBurstReg(char addr, char *buffer, char count)

{

  unsigned int i;



  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = addr | CC110L_WRITE_BURST; // Send address

  for (i = 0; i < count; i++)

  {

    while (!(IFG2&UCB0TXIFG));              // Wait for TXBUF ready

    UCB0TXBUF = buffer[i];                  // Send data

  }

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  CSn_OUT |= CSn_PIN;         // /CS disable

}



char SPIReadReg(char addr)

{

  char x;



  CSn_OUT &= ~CSn_PIN;               // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = (addr | CC110L_READ_SINGLE);// Send address

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = 1;                            // Dummy write so we can read data

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  x = UCB0RXBUF;                            // Read data

  CSn_OUT |= CSn_PIN;         // /CS disable

  return x;



}



void SPIReadBurstReg(char addr, char *buffer, char count)

{

  char i;

  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = (addr | CC110L_READ_BURST);// Send address

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  for (i = 0; i < count; i++)

  {

    UCB0TXBUF = 1;                          //Initiate next data RX, meanwhile..

	while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

    buffer[i] = UCB0RXBUF;                  // Store data from last data RX

  }

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  CSn_OUT |= CSn_PIN;         // /CS disable

}



char SPIReadStatus(char addr)

{

  char status;

  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = (addr | CC110L_READ_BURST);// Send address

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = 0X01;                            // Dummy write so we can read data

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  status = UCB0RXBUF;                       // Read data

  CSn_OUT |= CSn_PIN;         // /CS disable

  return status;

}





char SPIReadStatusByte()

{

  char status;



  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = 0x01;                         // Send dummy var

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  status = UCB0RXBUF;                       // Read data

  CSn_OUT |= CSn_PIN;         // /CS disable

  return status;

}



void SPIStrobe(char strobe)

{

  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while((GDO2_IN & GDO2_PIN));       // Wait for chip ready

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = strobe;                       // Send strobe

  // Strobe addr is now being TX'ed

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  CSn_OUT |= CSn_PIN;         // /CS disable

}



void PowerupResetCC110L(void)

{

  CSn_OUT |= CSn_PIN;

  Wait(60);

  CSn_OUT &= ~CSn_PIN;

  Wait(60);

  CSn_OUT |= CSn_PIN;

  Wait(60);

  CSn_OUT &= ~CSn_PIN;        // /CS enable

  while (!(IFG2&UCB0TXIFG));                // Wait for TXBUF ready

  UCB0TXBUF = CC110L_SRES;               // Send strobe

  // Strobe addr is now being TX'ed

  while (UCB0STAT & UCBUSY);                // Wait for TX to complete

  CSn_OUT |= CSn_PIN;         // /CS disable

}





//GFSK_38Kbps_18KHz_70KHzRxBW_-0.5dBm

void setup_cc110l(void)

{

PowerupResetCC110L();

writeRFSettings();

SPIStrobe(CC110L_SRX);



}

void writeRFSettings(void)

{

    // Write register settings

    SPIWriteReg(CC110L_IOCFG2,   0x29); // GDO2 output pin config.`CHIP_RDYn
    SPIWriteReg(CC110L_IOCFG1,   0x2E); // GDO1 output pin config.`HIGH IMP
    SPIWriteReg(CC110L_IOCFG0,   0x06); // GDO0 output pin config.`high on rx and tx
    SPIWriteReg(CC110L_FIFOTHR,  0x07); // THR.`32 bytes RX ; 33 bytes TX fifo
    SPIWriteReg(CC110L_SYNC1,    0xD3); // Packet automation control.`
    SPIWriteReg(CC110L_SYNC0,    0x91); // Packet automation control.`
    SPIWriteReg(CC110L_PKTLEN,   0xFF); // Packet length.`
    SPIWriteReg(CC110L_PKTCTRL1, 0x04); // Packet automation control.`flush when CRC not ok
    SPIWriteReg(CC110L_PKTCTRL0, 0x05); // Packet automation control.`normal mode ,CRC enabled , variable packet length
    SPIWriteReg(CC110L_ADDR,     0x00); // Device address.`
    SPIWriteReg(CC110L_CHANNR,   0x00); // Channel number.
    SPIWriteReg(CC110L_FSCTRL1,  0x06); // Freq synthesizer control.`
    SPIWriteReg(CC110L_FSCTRL0,  0x00); // Freq synthesizer control.`
    SPIWriteReg(CC110L_FREQ2,    0x20); // Freq control word, high byte`
    SPIWriteReg(CC110L_FREQ1,    0x25); // Freq control word, mid byte.`
    SPIWriteReg(CC110L_FREQ0,    0xED); // Freq control word, low byte.`
    SPIWriteReg(CC110L_MDMCFG4,  0xEA); // Modem configuration.`
    SPIWriteReg(CC110L_MDMCFG3,  0x71); // Modem configuration.`
    SPIWriteReg(CC110L_MDMCFG2,  0x13); // Modem configuration.`
    SPIWriteReg(CC110L_MDMCFG1,  0x20); // Modem configuration.`
    SPIWriteReg(CC110L_MDMCFG0,  0xF8); // Modem configuration.`
    SPIWriteReg(CC110L_DEVIATN,  0x33); // Modem dev (when FSK mod en)`??
    SPIWriteReg(CC110L_MCSM2,    0x07); //MainRadio Cntrl State Machine`
    SPIWriteReg(CC110L_MCSM1 ,   0x3F); //MainRadio Cntrl State Machine`stay in tx
    SPIWriteReg(CC110L_MCSM0 ,   0x18); //MainRadio Cntrl State Machine`
    SPIWriteReg(CC110L_FOCCFG,   0x16); // Freq Offset Compens. Config`??
    SPIWriteReg(CC110L_BSCFG,    0x6C); //  Bit synchronization config.`??
    SPIWriteReg(CC110L_AGCCTRL2, 0x43); // AGC control.`
    SPIWriteReg(CC110L_AGCCTRL1, 0x4F); // AGC control.`
    SPIWriteReg(CC110L_AGCCTRL0, 0x91); // AGC control.`
    SPIWriteReg(CC110L_FREND1,   0x56); // Front end RX configuration.`
    SPIWriteReg(CC110L_FREND0,   0x10); // Front end RX configuration.`
    SPIWriteReg(CC110L_FSCAL3,   0xE9); // Frequency synthesizer cal.`
    SPIWriteReg(CC110L_FSCAL2,   0x2A); // Frequency synthesizer cal.`
    SPIWriteReg(CC110L_FSCAL1,   0x00); // Frequency synthesizer cal.`
    SPIWriteReg(CC110L_FSCAL0,   0x1F); // Frequency synthesizer cal.`
    SPIWriteReg(CC110L_TEST2,    0x81); // Various test settings.`
    SPIWriteReg(CC110L_TEST1,    0x35); // Various test settings.`
    SPIWriteReg(CC110L_TEST0,    0x09); // Various test settings.`
}



// PATABLE (-0.5 dBm output power)
extern char paTable[] = {0x70};   // -0.5 dBm
extern char paTableLen = 1;



//-----------------------------------------------------------------------------

//  void RFSendPacket(char *txBuffer, char size)

//

//  DESCRIPTION:

//  This function transmits a packet with length up to 63 bytes.  To use this

//  function, GD00 must be configured to be asserted when sync word is sent and

//  de-asserted at the end of the packet, which is accomplished by setting the

//  IOCFG0 register to 0x06, per the CCxxxx datasheet.  GDO0 goes high at

//  packet start and returns low when complete.  The function polls GDO0 to

//  ensure packet completion before returning.

//

//  ARGUMENTS:

//      char *txBuffer

//          Pointer to a buffer containing the data to be transmitted

//

//      char size

//          The size of the txBuffer

//-----------------------------------------------------------------------------



void sendbyte(char a)

{

char Buffer[3];



Buffer[0] = 2;                           // Packet length

Buffer[1] = 0x01;                        // Packet address

Buffer[2] = a;    					 // Load four switch inputs

RFSendPacket(Buffer, 3);                 // Send value over RF

}



void RFSendPacket(char *txBuffer, char size)

{

  GDO0_IE &= ~GDO0_PIN;          // Disable interrupt

  SPIWriteBurstReg(CC110L_TXFIFO, txBuffer, size); // Write TX data

  SPIStrobe(CC110L_STX);           // Change state to TX, initiating data transfer

  while (!(GDO0_IN & GDO0_PIN));// Wait GDO0 to go hi -> sync TX'ed

  while (GDO0_IN&GDO0_PIN);   // Wait GDO0 to clear -> end of pkt

  GDO0_IFG &= ~GDO0_PIN;      // After pkt TX, this flag is set.

                                            // Has to be cleared before existing

  GDO0_IFG &= ~GDO0_PIN;

  GDO0_IE |= GDO0_PIN;            // Enable interrupt



}







//-----------------------------------------------------------------------------

//  char RFReceivePacket(char *rxBuffer, char *length)

//

//  DESCRIPTION:

//  Receives a packet of variable length (first byte in the packet must be the

//  length byte).  The packet length should not exceed the RXFIFO size.  To use

//  this function, APPEND_STATUS in the PKTCTRL1 register must be enabled.  It

//  is assumed that the function is called after it is known that a packet has

//  been received; for example, in response to GDO0 going low when it is

//  configured to output packet reception status.

//

//  The RXBYTES register is first read to ensure there are bytes in the FIFO.

//  This is done because the GDO signal will go high even if the FIFO is flushed

//  due to address filtering, CRC filtering, or packet length filtering.

//

//  ARGUMENTS:

//      char *rxBuffer

//          Pointer to the buffer where the incoming data should be stored

//      char *length

//          Pointer to a variable containing the size of the buffer where the

//          incoming data should be stored. After this function returns, that

//          variable holds the packet length.

//

//  RETURN VALUE:

//      char

//          0x80:  CRC OK

//          0x00:  CRC NOT OK (or no pkt was put in the RXFIFO due to filtering)

//-----------------------------------------------------------------------------

char RFReceivePacket(char *rxBuffer, char *length)

{

  char status[2];

  char pktLen;



  if ((SPIReadStatus(CC110L_RXBYTES) & CC110L_NUM_RXBYTES))

  {

		pktLen = SPIReadReg(CC110L_RXFIFO); // Read length byte

		if (pktLen <= *length)                  // If pktLen size <= rxBuffer

		{

		  SPIReadBurstReg(CC110L_RXFIFO, rxBuffer, pktLen); // Pull data

		  *length = pktLen;                     // Return the actual size

		  SPIReadBurstReg(CC110L_RXFIFO, status, 2);

												// Read appended status bytes

		  return (char)(status[CC110L_LQI_RX]&CC110L_CRC_OK);

		}                                       // Return CRC_OK bit

		else

		{

		  *length = pktLen;                     // Return the large size

		  SPIStrobe(CC110L_SFRX);      // Flush RXFIFO

		  return 0;                             // Error

		}

  }

  else
      return 0;                             // Error
}



